#!/usr/bin/python

"""
Note hylafax
------------

Fax in ricezione:

	Alla fine di ogni ricezione viene eseguito direttamente da hylafax:
	bin/faxrcvd "recvq/fax000000235.tif" "ttyS1" "000000509" "" "" ""
	(ad esempio)

	Di conseguenza questo script e' chiamato dal file bin/faxrcvd (bash
	script, ultima riga, l'unica modifica fatta rispetto alla versione
	orginale hylafax).

	Recuperando da bin/faxrcvd solo la variabile $COMMID si esegue:
	bin/hylafax.py -v -s django.settings -i -c $COMMID

Fax in invio:

	Hylafax si comporta (molto) diversamente per i fax in invio.
	Ad ogni tentativo di invio viene creato un file di log in log/ .

	Soltanto quando l'invio e' terminato viene chiamato lo script notify
	con unico argomento il file contenente le informazioni sull'invio.
	Questo file e' built-in di hylafax e risiede nella directory doneq/ 
	nel formato q$JOBID (esempio doneq/q890).

	Nonostante questo file contenga gia' numerose informazioni sul fax in
	questione, si preferisce recuperarne solo il commid per poi parsare
	il log 'grezzo' in log/ cosi' come succede per i fax ricevuti.

Exit Status:

	Passato da bin/notify (solo fax inviati) puo' essere:
	    Killed = job interrotto dall'utente con faxrm
	    done = fax inviato
	    failed = fax non inviato
"""

USAGE ="""
  usage: %prog [options]
  -s, --settings=file: settings file eg: jobs.settings
  -d, --dryrun: don't really touch the db
  -v, --verbose: can't you immagine?
  -q, --queue=id: relative queue file path (outgoing fax, ex doneq/q890)
  -c, --commid=id: commid (incoming fax, ex c000003174)
  -r, --root=root: set root dir
  -x, --exit=exit: exit status passed by hylafax
"""

from jumbo.utils import optionparse
import os, os.path, sys, re, commands
from datetime import datetime
import datetime as d

opt, args = optionparse.parse(USAGE)
os.environ['DJANGO_SETTINGS_MODULE'] = opt.settings
from django.conf import settings
from django.core.mail import EmailMessage

from fax.models import Fax

#HYLA_ROOT  = '/var/spool/hylafax/'
HYLA_ROOT  = '/home/marco/Documents/fax/'

if opt.root:
    HYLA_ROOT = opt.root
    print "Attenzione root impostata a " + opt.root

LOG_DIR    = HYLA_ROOT + 'log/'
RECV_DIR   = HYLA_ROOT + 'recvq/'
SENT_DIR   = HYLA_ROOT + 'docq/'
PDF_DIR    = settings.FAX_PATH_DOCUMENTS

DATE_FORMAT='%b %d %H:%M:%S'
today = d.date.today().strftime('%y%m%d')
rec_filename=''

PATTERN_OUT_STATUS = "(status)?(error)?"
out_status = re.compile(PATTERN_OUT_STATUS)

PATTERN_IN_RECV = ".{15,15}.*RECV FAX \(.*\)\: recvq\/(?P<file>.*) from (?P<sender>.*)\,.*, (?P<pages>.*) pages? in (?P<duration>.*)"
PATTERN_IN_RECIPIENT = ".{15,15}.*SESSION BEGIN.* .* (?P<recipient_number>.*)"
PATTERN_IN_DEVICE = ".{15,15}.*recvq/fax\d+.tif.*(?P<device>tty\w+?\d+?)\" "
PATTERN_IN_SENDER = ".{15,15}.*CallID.* '(?P<sender_number>.*)' '.*' '.*' '.*'$"

in_recv = re.compile(PATTERN_IN_RECV)
in_recipient = re.compile(PATTERN_IN_RECIPIENT)
in_device = re.compile(PATTERN_IN_DEVICE)
in_sender = re.compile(PATTERN_IN_SENDER)

DATE_FORMAT='%b %d %H:%M:%S'

MAIL_BODY_SENT = """
Destinatario: %s
Oggetto del Fax: %s
ID Lavoro: %s
Data e ora spedizione: %s
Conferma Invio: %s
Tempo di spedizione: %s
Pagine inviate: %s
"""

MAIL_BODY_RCV = """
Mittente: %s
Numero mittente: %s
ID Lavoro: %s
Data e ora ricezione: %s
Tempo ricezione: %s
Pagine ricevute: %s
"""

def get_fax():
    #if opt.queue:
    #    fax = Fax.objects.get(job=opt.queue.split('/q').pop())
    #else:
    fax = Fax()
    return fax

def parse_log(fax):
    print "ENTRO PARSE LOG"
    if opt.queue: # viene passato es. doneq/q10
        log_file_name = os.path.join(HYLA_ROOT, opt.queue)
    else: # viene passato es. c0000010
        log_file_name = os.path.join(LOG_DIR, opt.commid)
    fax.date_create = datetime.fromtimestamp(os.path.getmtime(log_file_name))
    log = open(log_file_name)

    print "Log file name: %s " % log_file_name
    fax.date_last_modify = datetime.fromtimestamp(os.path.getmtime(log_file_name))
    if opt.queue:
        info = get_info_from_queue(log)
        fax.recipient_number = info['number']
        if opt.exit == 'killed':
            fax.job = ''
            fax.duration = ''
        else:
            fax.job = 'c' + info['commid']
            if info['commid']:
                duration, sender = get_time_and_sender(info['commid'])
            else:
                duration = sender = None
            fax.duration = duration
            fax.sender = sender
        fax.pages = info['totpages']
        
        if opt.exit == 'done': 
            fax.status = 4
        else:
            fax.status = 5
            fax.error = info['status'] or 'Killed'
    else:
        for line in log:
            parse_raw_line(line)


def get_time_and_sender(filename):
    filename = "c%s" % (filename,)
    f = open(os.path.join(LOG_DIR, filename))
    sender = duration = None
    
    for line in f:
        m = re.search('SEND FAX.*FROM.*docq.*sent in (?P<time>.*)\)', line)
        if m:
            duration = m.group('time')
            
        m = re.search('SEND FAX: JOB.*FROM.*@(?P<sender>.*)>.*$', line)
        if m:
            sender = m.group('sender')
    
    return duration, sender


def get_info_from_queue(log):
    info = {}
    for line in log:
        line = line.rstrip('\n')
        try:
            key = line[:line.find(':')]
            val = line[line.find(':')+1:]
            info[key] = val
        except:
            pass
    return info

# FAX OK
# sdest         #### destinatario
# stty          #### device modem in uscita
# ssend         #### numero mittente
#commid
#totpages

# scommid       #### id log fax (c000000001)
# stime         #### data e ora tentativo spedizione
# sdest         #### destinatario
# ssend         #### mittente
# stty          #### device modem in uscita
# serror        #### errore del fax


def parse_raw_line(line):
    m = in_recv.match(line)
    if m:    	
        global rec_filename
        rec_filename = m.group('file')
        #c = m.group('sender')
        #tmp = c
        #for char in ('.', ' ', '|', '-', '/', '\\'):
        #    tmp = tmp.replace(char, '')
        #tmp = tmp.replace('+','00')
        #if tmp.isdigit():
        #    fax.sender_number = tmp[:16]
        #    fax.sender = c[:100]
        #else:
        fax.sender = m.group('sender')
        fax.pages = m.group('pages')
        fax.duration = m.group('duration')

    m = in_recipient.search(line)
    if m:
        fax.recipient_number = m.group('recipient_number')

    m = in_device.search(line)
    if m:
        fax.device = m.group('device')

    m = in_sender.search(line)
    if m:
        fax.sender_number = m.group('sender_number')

    fax.job = opt.commid
    fax.cover = False
    fax.status = 2
    fax.subject = 'fax ricevuto'
    fax.read = 0

fax = get_fax()
parse_log(fax)

# se fax inviato, la conferma invio la si recupera da fax.status
if fax.status == 4:
    mail_status = 'OK'
else:
    mail_status = fax.error

for i in fax._meta.fields:
    print i.name, getattr(fax, i.name)
    #print fax.status

if not opt.dryrun:
    from django.conf import settings

    fax.save()
    if opt.commid:
        log_file_name = os.path.join(LOG_DIR, opt.commid)
        fax.date_create = datetime.fromtimestamp(os.path.getmtime(log_file_name))
        fax.date_last_modify = datetime.fromtimestamp(os.path.getmtime(log_file_name))

        fax.file = '%s%s_%s.pdf' % (PDF_DIR, today, fax.id)
        fax.save()
        cmd = "umask 022; tiff2pdf -o %s %s%s" % (fax.file.path, RECV_DIR, rec_filename)
        #cmd = "umask 022; tiff2ps -a %s%s | /usr/bin/ps2pdf - %s" % (RECV_DIR, rec_filename, fax.file.path)
        print "eseguo " + cmd
        status, output = commands.getstatusoutput(cmd)
    elif opt.queue:
        n = opt.queue[-3:]
        log_file_name = os.path.join(HYLA_ROOT, opt.queue)
        fax.subject = 'fax inviato'
        fax.date_create = datetime.fromtimestamp(os.path.getmtime(log_file_name))
        fax.date_last_modify = datetime.fromtimestamp(os.path.getmtime(log_file_name))
        fax.file = 'doc%s.pdf' % (n,)
        fax.save()
        cmd = "umask 022; /usr/bin/ps2pdf %sdoc%s.ps %sdoc%s.pdf" % (SENT_DIR, n, PDF_DIR, n)
        #cmd = "umask 022; tiff2ps -a %s%s | /usr/bin/ps2pdf - %s" % (RECV_DIR, rec_filename, fax.file.path)
        print "eseguo " + cmd
        status, output = commands.getstatusoutput(cmd)

    info = {'operator'  : fax.sender_number,
            'pdf_dir'   : PDF_DIR,
            'file'      : fax.file,
            }
    
    data_fax = fax.date_last_modify.strftime('%d/%m/%Y %H:%M:%S')
